CON

  _clkmode = xtal1 + pll16x                                                      
  _xinfreq = 5_000_000

    'I2C pins
  SCL_PIN  =  0'20
  SDA_PIN  =  1'21

  BITRATE = 100_000  'I2C @ 100k
   
  HTU21D_ADDRESS = $40 ' 7-bit I2C address '%100_0000

  DEW_POINT_A = 8.1332
  DEW_POINT_B = 1762.39
  DEW_POINT_C = 235.66

  DEFAULT_TEMPERATURE = 22
  
  BAUD = 115_200
  
OBJ
  
  I2c : "I2C PASM driver v1.8od"  
  F32 : "Float32"
  Pst : "Parallax Serial Terminal"
  Format : "StrFmt"

    
PUB Main | rawTemperature, atemperature, temperature, ftemperature, rawHumidity, relativeHumidity, fRelativeHumidity, {
  }fPartialPressure, fDewPoint, astring[4]

  rawTemperature := DEFAULT_TEMPERATURE
  
  Pst.Start(BAUD)
  waitcnt(clkfreq / 2 + cnt)
  
{  repeat
    result := Pst.RxCount
    Pst.Str(string(11, 13, "Press any key to start."))
    waitcnt(clkfreq / 4 + cnt)
  until result
}   
  Pst.RxFlush
   
  Pst.Clear

  F32.Start
    
  I2C.Start(SCL_PIN, SDA_PIN, BITRATE)
  
  Pst.Str(string("user register bits =  "))
  I2c.Command(HTU21D_ADDRESS, $E7)
  ReadableBin(I2C.ReadNext(HTU21D_ADDRESS), 8, 4)

  waitcnt(clkfreq * 2 + cnt)

  repeat
    Pst.ClearEnd
    Pst.NewLine
    Pst.ClearBelow
    Pst.Home
    Pst.Str(string(11, 13, "HTU21D Dew Point Demo"))
    Pst.Str(string(11, 13, "msb + lsb (rawHumidity)    = "))
    rawHumidity := I2C.readWordB(HTU21D_ADDRESS, $E5)
    Pst.Dec(rawHumidity)
    Pst.Str(string(" = "))
    ReadableBin(rawHumidity, 16, 4)
     
    relativeHumidity := RawToRhInteger(rawHumidity)
    fRelativeHumidity := RawToRhFloat(rawHumidity)
    Pst.Str(string(11, 13, "relativeHumidity Integer Math        =     "))
    Pst.Dec(relativeHumidity)
    Pst.Str(string(11, 13, "relativeHumidity Floating Point Math = "))
    DisplayFloat(fRelativeHumidity, 2)

 '   
    Pst.NewLine
    Pst.Str(string(11, 13, "msb + lsb (rawTemperature) = ")) 
    rawTemperature := I2C.readWordB(HTU21D_ADDRESS,$E3)
    Pst.dec(rawTemperature)
    Pst.Str(string(" = "))
    ReadableBin(rawTemperature, 16, 4)
    
    temperature := rawTemperature * 175 / 65536 - 46
    ftemperature:=F32.FSub(F32.FDiv(F32.FMul(F32.FFloat(rawTemperature),{
    }175.72),65536.0),46.85)

    Pst.Str(string(11, 13, "temperature Integer Math             =     "))
    Pst.Dec(temperature)
    Pst.Str(string(11, 13, "temperature Floating Point Math      = "))
    DisplayFloat(ftemperature, 2)

'
    Pst.NewLine
    Pst.NewLine
'    astring := $00_44_43_42_41'_32_31_30_40
    astring := $45_44_43_42'_41
    'Pst.Str(strsize(@astring))                                
    Pst.NewLine
    Pst.Str(@astring)
    Pst.NewLine
    Pst.Dec(@astring)
    Pst.NewLine
    Pst.Dec(astring)
    Pst.NewLine
    Pst.Dec(strsize(@astring))
    Pst.NewLine

    fPartialPressure := GetPartialPressure(ftemperature)
    
    Pst.Str(string(11, 13, "Partial Pressure = "))
    DisplayFloat(fPartialPressure, 2)

    fDewPoint := GetDewPoint(fRelativeHumidity, fPartialPressure)
    Pst.Str(string(11, 13, "Dew Point        = "))
    DisplayFloat(fDewPoint, 2)

    waitcnt(clkfreq + cnt)

PUB GetDewPoint(fRelativeHumidity, fPartialPressure)

  result := F32.FNeg(F32.FAdd(F32.FDiv(DEW_POINT_B, F32.FSub(F32.Log10(F32.FMul( {
  } fRelativeHumidity, F32.FDiv(fPartialPressure, 100.0))), DEW_POINT_A)), DEW_POINT_C))

PUB GetPartialPressure(temperature) | fTemperature

  fTemperature := F32.FFloat(temperature)
  ftemperature := temperature

  result := F32.Exp10(F32.FSub(DEW_POINT_A, F32.FDiv(DEW_POINT_B, F32.FAdd(fTemperature, {
  } DEW_POINT_C))))
  
PUB RawToRhInteger(rawRhValue)

  result := (rawRhValue * 125 / 65536) - 6

PUB RawToRhFloat(rawRhValue)
  
  result := F32.FSub(F32.FDiv(F32.FMul(F32.FFloat(rawRhValue), 125.0), 65536.0), 6.0)

PRI DisplayFloat(fValue, decimalPlaces) | fScale

  fScale := F32.Exp10(F32.FFloat(decimalPlaces))
  result := RoundScaled(fValue, fScale)
  DecPoint(result, decimalPlaces)
  
PRI RoundScaled(fValue, fScale)

  result := F32.FRound(F32.FMul(fValue, fScale))
  
PRI DecPoint(value, places) | tempBuffer[4]

  result := Format.FDec(@tempBuffer, value, 9, places)
'  byte[result] := 0
  Pst.Str(@tempBuffer)

PUB ReadableBin(localValue, size, groupSize) | bufferPtr, localBuffer[12]    
'' This method display binary numbers
'' in easier to read groups.

  bufferPtr := Format.Ch(@localBuffer, "%")
  
  result := size // groupSize
   
  if result
    size -= result
    bufferPtr := Format.Bin(bufferPtr, localValue >> size, result)
    if size
      bufferPtr := Format.Ch(bufferPtr, "_")
      
  size -= groupSize
 
  repeat while size => 0
    'Debug.Bin(localValue >> size, 4)
    bufferPtr := Format.Bin(bufferPtr, localValue >> size, groupSize)
    if size
      bufferPtr := Format.Ch(bufferPtr, "_")  
    size -= groupSize
    
  byte[bufferPtr] := 0  
  Pst.Str(@localBuffer)